<?php
/***********************************************************
 *			db_data.php
 *		    -------------------
 * Created		: Sat, oct 30, 2005
 * Copyright		: (C) LookStrike Team
 * WebSite		: http://www.lookstrike.com
 *
 * File Owner: jsgoupil
 * $Id: db_data.php,v 1.13 2008/01/29 20:48:53 jsgoupil Exp $
 *
 ***********************************************************/
/**
 * Parent class of database data which allows to make global operations on the
 * classes.
 * This class is abstract and should not be instanciated directly.
 */
abstract class db_data {
	const ERROR_NONE		= 0;
	const ERROR_NOTALLOWED		= -1;
	const ERROR_FIELDMISSING	= -2;
	const ERROR_FIELDERROR		= -3;
	const ERROR_UNIQUE		= -4;
	// Watch out, the index are signed in array
	const ERROR_SQL			= 0x0ffffffe;
	const ERROR_UNKNOWN		= 0x0fffffff;

	const OPERATION_INSERT		= 1;
	const OPERATION_MODIFY		= 2;
	const OPERATION_DELETE		= 3;

	const DB_TIME_DATE		= 1;
	const DB_TIME_TIMESTAMP		= 2;
	const DB_TIME_UNIX		= 3;

	protected $_ERROR_LANGUAGE;
	private $_ERROR_LANGUAGE_DEFAULT = array(
		self::ERROR_NONE		=> 'NONE',
		self::ERROR_NOTALLOWED		=> 'NOTALLOWED',
		self::ERROR_FIELDMISSING	=> 'FIELDMISSING',
		self::ERROR_FIELDERROR		=> 'FIELDERROR',
		self::ERROR_UNIQUE		=> 'UNIQUE',
		self::ERROR_SQL			=> 'SQL',
		self::ERROR_UNKNOWN		=> 'UNKNOWN'
	);

	private $_error;
	private $_admin = false; // Auto Not-Admin
	protected $_primary;
	protected $_unique;
	protected $_required_admin;
	protected $_required;
	protected $_filter;
	protected $_checkbox;

	public abstract function insert();
	public abstract function modify();
	public abstract function delete();

	/**
	 * Returns the error language associated with the $code
	 *
	 * @param int $code
	 * @return string
	 */
	public function getErrorLanguage($code) {
		global $language_db;

		if (isset($this->_ERROR_LANGUAGE[$code])) {
			return $language_db['ERROR'][$this->_ERROR_LANGUAGE[$code]];
		} elseif (isset($this->_ERROR_LANGUAGE_DEFAULT[$code])) {
			return $language_db['ERROR'][$this->_ERROR_LANGUAGE_DEFAULT[$code]];
		}
		return $language_db['ERROR']['UNKNOWN'];
	}

	/**
	 * Fixes if you are admin or not
	 *
	 * @param bool $isAdmin
	 */
	public function setAdmin($isAdmin) {
		$this->_admin = (bool)$isAdmin;
	}

	/**
	 * Checks if the current data in the class are existing in the database.
	 * Returns true if it exists, otherwise false.
	 *
	 * @return bool
	 */
	protected function exists() {
		global $db;

		$req = $db->sql_query('SELECT ' . self::createSelect($this->_primary) . ' FROM ' . $this->getTableName($this) . ' WHERE ' . $this->createWhere($this->_primary));

		return ($db->sql_numrows($req) === 1) ? true : false;
	}

	/**
	 * Generates insert code with multi-arguments.
	 * ($key1, $key2, $key3, ...) VALUES ($this->$key1, $this->$key2, $this->$key3, ...)
	 * These keys can use function: specify the argument as an array. $key1 has
	 * to be the key of the array and the value has to be the "function name".
	 * See _generate() for more explanation.
	 *
	 * @param mixed ...
	 * @return string
	 */
	protected function generateInsert() {
		$string1 = '';
		$string2 = '';
		$args = func_get_args();
		$c = count($args);
		for ($i = 0; $i < $c; $i++) {
			if (is_array($args[$i])) {
				$string1 .= '`' . key($args[$i]) . '`, ';
				$string2 .= $this->_generate($args[$i]);
			} else {
				$string1 .= '`' . $args[$i] . '`, ';
				$string2 .= '\'' . addslashes($this->$args[$i]) . '\', ';
			}
		}
		$string1 = substr($string1, 0, -2);
		$string2 = substr($string2, 0, -2);

		$string = '(' . $string1 . ') VALUES (' . $string2 . ')';
		return $string;
	}

	/**
	 * Generates modify code with multi-arguments.
	 * $key1=$this->$key1, $key2=$this->$key2, $key3=$this->key3
	 * These keys can use function: specify the argument as an array. $key1 has
	 * to be the key of the array and the value has to be the "function name".
	 * See _generate() for more explanation.
	 *
	 * @param mixed ...
	 * @return string
	 */
	protected function generateModify() {
		$string = '';
		$args = func_get_args();
		$c = count($args);
		for ($i = 0; $i < $c; $i++) {
			if (is_array($args[$i])) {
				$string .= '`' . key($args[$i]) . '`=' . $this->_generate($args[$i]);
			} else {
				$string .= '`' . $args[$i] . '`=\'' . addslashes($this->$args[$i]) . '\', ';
			}
		}
		$string = substr($string, 0, -2);
		return $string;
	}

	/**
	 * Used to modify the value of SQL statement.
	 * The key of the array is the code for the member variable
	 * and the value is the function name.
	 *
	 * @param array $array
	 * @return string
	 */
	private function _generate($array) {
		global $db;

		$key = key($array);
		$val = current($array);

		switch ($val) {
			case 'same':
				return '`' . $key . '`, ';
			case 'null':
				return 'NULL, ';
			case 'ifnull':
				return (!empty($this->$key)) ? '\'' . addslashes($this->$key) . '\', ' : 'NULL, ';
			case 'now':
				return 'NOW(), ';
			case 'md5':
				// More tricky because we can't md5 an md5 form !
				// So we have to search in database if the password has changed
				// If so, we will md5 it... otherwise, we won't change it
				$req = $db->sql_query('SELECT ' . $key . ' FROM ' . $this->getTableName($this) . ' WHERE ' . $this->createWhere($this->_primary));
				$line = $db->sql_numrows($req);
				if ($line === 1) {
					$data = $db->sql_fetchrow($req);
					if ($data[$key] === $this->$key) {
						return '`' . $key . '`, ';
					}
				}
				return '\'' . md5($this->$key) . '\', ';
			default:
				return '\'' . addslashes($this->$key) . '\', ';
		}
	}

	/**
	 * Fetches the more detailed error explanation.
	 *
	 * @return string
	 */
	public function getError() {
		return $this->_error;
	}

	/**
	 * If passing only a string, returning : $key=\'$this->$key\'
	 * Otherwise, returning : $key[0]=\'$this->$key[0]\' AND $key[1]=\'$this->key[1]\' AND ...
	 *
	 * @param mixed $key
	 * @return string
	 */
	protected function createWhere($key) {
		// If it is not an array, we will array it to use the same next code
		if (!is_array($key)) {
			$key = array($key);
		}

		$string = '';
		$c = count($key);
		for ($i = 0; $i < $c; $i++) {
			$string .= '`' . $key[$i] . '`=\'' . addslashes($this->$key[$i]) . '\' AND ';
		}
		$string = substr($string, 0, -5);

		return $string;
	}

	/**
	 * If passing only a string, returning : $key
	 * Otherwise, returning : $key[0], $key[1], ...
	 *
	 * @param mixed $key
	 * @return string
	 */
	protected static function createSelect($key) {
		// If it is not an array, we will array it to use the same next code
		if (!is_array($key)) {
			$key = array($key);
		}

		$string = '';
		$c = count($key);
		for ($i = 0; $i < $c; $i++) {
			$string .= $key[$i] . ', ';
		}
		$string = substr($string, 0, -2);

		return $string;
	}

	/**
	 * Compare $this->$key[0] with $data[$key[0]], $this->$key[1] with $data[$key[1]]
	 * If one of them is not the same, return false. Otherwise, return true
	 * $key should be $_primary
	 * $data should be $data from sql_fetchrow
	 *
	 * @param mixed $key
	 * @param string[]$data
	 * @return bool
	 */
	protected function comparePrimary($key, $data) {
		// If it is not an array, we will array it to use the same next code
		if (!is_array($key)) {
			$key = array($key);
		}

		$array = array();
		$c = count($key);
		for ($i = 0; $i < $c; $i++) {
			if ($this->$key[$i] != $data[$key[$i]]) {		// Leave weak check
				return false;
			}
		}

		return true;
	}

	/**
	 * By calling this method, the data will be checked with
	 * require, unique and filter function.
	 *
	 * @param int $operation OPERATION_INSERT|OPERATION_MODIFY|OPERATION_DELETE
	 * @return int
	 */
	protected function checkData($operation) {
		$error = $this->checkRequired();
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}
		$error = $this->checkUnique($operation);
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}
		$error = $this->checkFilter();
		if ($error !== db_data::ERROR_NONE) {
			return $error;
		}

		return db_data::ERROR_NONE;
	}

	/**
	 * Checks all the required data with the _require member.
	 * If admin is set to true, the _require_admin is used instead.
	 *
	 * @return int
	 */
	private function checkRequired() {
		if ($this->_admin === true) {
			$table = $this->_required_admin;
		} else {
			$table = $this->_required;
		}
		$this->_error = array();
		foreach ($table as $member) {
			$val = trim($this->$member);
			if (empty($val)) {
				$this->_error[] = $member;
			}
		}
		if (count($this->_error) > 0) {
			return self::ERROR_FIELDMISSING;
		}
		return self::ERROR_NONE;
	}

	/**
	 * Checks if the data in _unique are unique in the database.
	 * The _unique can be a string or a table containing unique keys.
	 * If a table is provided, the key are "joined" together (like 2 primary keys)
	 * The operation state is needed to check if we are allowed to find ONE
	 * instance of the data or not.
	 *
	 * @param int $operation OPERATION_INSERT|OPERATION_MODIFY|OPERATION_DELETE
	 * @return int
	 */
	private function checkUnique($operation) {
		global $db;

		$this->_error = array();
		foreach ($this->_unique as $member) {
			$req = $db->sql_query('SELECT ' . self::createSelect($this->_primary) . ' FROM ' . $this->getTableName($this) . ' WHERE ' . $this->createWhere($member));
			if ($operation === self::OPERATION_INSERT && $db->sql_numrows($req) === 1) {
				$this->_error = $member;
				return self::ERROR_UNIQUE;
			} else if ($operation === self::OPERATION_MODIFY && $db->sql_numrows($req) === 1) {
				$data = $db->sql_fetchrow($req);
				if ($this->comparePrimary($this->_primary, $data) === false) {
					$this->_error[] = $member;
				}
			}
		}
		if (count($this->_error) > 0) {
			return self::ERROR_UNIQUE;
		}
		return self::ERROR_NONE;
	}

	/**
	 * This method applies a filter to the data to check their correctness.
	 * The method checks the _filter member. The key has to be the code member and
	 * the value has to be the "function name".
	 *
	 * @return int
	 */
	private function checkFilter() {
		$this->_error = array();
		foreach ($this->_filter as $key => $val) {
			switch ($val) {
				case 'date':
				if (checkdate(intval(substr($this->$key, 5, 2)), intval(substr($this->$key, 8, 2)), intval(substr($this->$key, 0, 4))) === false) {
					$this->_error[] = $key;
					return self::ERROR_FIELDERROR;
				}
				break;
				default:
			}
		}
		if (count($this->_error) > 0) {
			return self::ERROR_FIELDERROR;
		}
		return self::ERROR_NONE;
	}

	/**
	 * Returns the table you are supposed to use with your child class
	 *
	 * @param db_data $clazz
	 * @return string
	 */
	protected function getTableName($clazz) {
		global $sys_conf;

		if (is_object($clazz)) {
			$clazz = get_class($clazz);
		}
		$tablename = $sys_conf['db']['table_prefix'] . substr($clazz, 3);

		return $tablename;
	}

	/**
	 * Returns _primary variable
	 *
	 * @return mixed
	 */
	public function getPrimary() {
		return $this->_primary;
	}

	/**
	 * Returns _checkbox variable
	 *
	 * @return mixed
	 */
	public function getCheckbox() {
		return $this->_checkbox;
	}

	/**
	 * Returns if the class were successfully loaded.
	 * Returns true on success
	 *
	 * @return bool
	 */
	public function IsLoaded() {
		$primary = $this->_primary;
		if(!is_array($primary)) {
			$primary = array($primary);
		}

		$c = count($primary);
		for($i = 0; $i < $c; $i++) {
			if(intval($this->$primary[$i]) === 0) {
				return false;
			}
		}

		return true;
	}

	/**
	 * Recherche dans $tablename  l'aide du $whereclause fourni ainsi que le $orderclause.
	 * Le $whereclause, $orderclause et $limitclause peuvent tre fournis en texte ou en array.
	 * Le dernier paramtre $oneline permet d'effectuer un fetchrow simple si true ou un fetchrowset si false.
	 *
	 * @param string	$tablename
	 * @param mixed		$whereclause
	 * @param mixed		$orderclause
	 * @param mixed		$limitclause
	 * @param boolean	$oneline
	 * @return string[]
	 * @throw Exception SQL Erreur
	 */
	public static function searchFor($tablename, $whereclause, $orderclause = '', $limitclause = '', $oneline = false) {
		global $db, $sys_conf;

		$where = '';
		if(!empty($whereclause)) {
			$where = self::whereClause($whereclause, '');
		}

		$order = '';
		if(!empty($orderclause)) {
			$order = self::orderClause($orderclause);
		}

		$limit = '';
		if(!empty($limitclause)) {
			$limit = self::limitClause($limitclause);
		}

		$sql = 'SELECT * FROM '.$sys_conf['db']['table_prefix'].$tablename.$where.$order.$limit;
		$req = $db->sql_query($sql);
		$error = $db->sql_error();
		if(intval($error['code']) === 0) {	// Aucune Erreur dtecte
			if($db->sql_numrows($req) === 0) {
				return NULL;
			}
			if($oneline === true) {
				return $db->sql_fetchrow();
			} else {
				return $db->sql_fetchrowset();
			}
		} else {				// Une erreur dans la query s'est droule
			throw new Exception('<b>SQL Error :</b> '. $error['message'] . '<br />'.$sql);
			return false;
		}
	}

	/**
	 * Retourne un tableau de $clazzname duement rempli.
	 *
	 * @param string	$clazzname
	 * @param mixed		$whereclause
	 * @param mixed		$orderclause
	 * @param mixed		$limitclause
	 * @return $clazzname[]
	 * @throw Exception SQL Erreur
	 */
	public static function searchForClass($clazzname, $whereclause, $orderclause = '', $limitclause = '', $primaryclause = 'id') {
		global $db, $sys_conf;

		$tablename = substr($clazzname, 3);
		$where = '';
		if(!empty($whereclause)) {
			$where = self::whereClause($whereclause, '');
		}

		$order = '';
		if(!empty($orderclause)) {
			$order = self::orderClause($orderclause);
		}

		$limit = '';
		if(!empty($limitclause)) {
			$limit = self::limitClause($limitclause);
		}

		$primary = self::createSelect($primaryclause);

		$sql = 'SELECT '.$primary.' FROM '.$sys_conf['db']['table_prefix'].$tablename.$where.$order.$limit;
		$req = $db->sql_query($sql);
		$error = $db->sql_error();
		if(intval($error['code']) === 0) {	// Aucune Erreur dtecte
			$c = $db->sql_numrows($req);
			$data = $db->sql_fetchrowset();
			$ret = array();
			for($i = 0; $i < $c; $i++) {
				$primarywhere = self::primaryClauseWhere($primaryclause, $data[$i]);
				$clazz = new $clazzname;
				$clazz->fill($primarywhere);
				$ret[] = $clazz;
			}

			return $ret;
		} else {				// Une erreur dans la query s'est droule
			throw new Exception('<b>SQL Error :</b> '. $error['message'] . '<br />'.$sql);
			return false;
		}
	}

	/**
	 * Rempli la classe  partir de la table associe  celle ci.
	 * L'argument d'entr serviera pour une clause WHERE
	 * La valeur de l'argument peut tre soit une chane de caractres normal
	 *
	 * Elle peut tre aussi un tableau contenant en cl le champ et la valeur du champ
	 * en valeur de la cl. Pour faire un AND ou un OR, il suffit d'entourer les
	 * cls et valeurs par un tableau contenant en cl 'AND' ou 'OR'.
	 * Exemple :
	 * array('AND'=>array('champ1'=>1, 'champ2'=>2));
	 *
	 * La fonction qui analyse la clause where est rcursive, il peut donc y avoir
	 * des clauses AND et OR imbriques.
	 * La cl peut commencer par ! pour effectuer une comparaision != au lieu de =.
	 *
	 * Le $orderclause peut tre pass directement en string ou en
	 * array ayant comme cl la valeur  classer et en valeur la manire du
	 * classement (DESC ou ASC). Les valeurs  classer doivent tre
	 * places dans l'ordre pour le classement.
	 *
	 * Le $limitclause peut tre pass directement en string ou en
	 * array ayant comme premire valeur le nombre d'affichage et
	 * comme deuxime valeur, l'index.
	 *
	 * @param mixed $whereclause
	 * @param mixed $orderclause
	 * @param mixed $limitclause
	 * @return int
	 * @throw Exception SQL Erreur
	 */
	public function fill($whereclause = '', $orderclause = '', $limitclause = '') {
		global $db, $sys_conf;

		$tablename = substr(get_class($this), 3);
		$where = '';
		if(!empty($whereclause)) {
			$where = self::whereClause($whereclause, '');
		}

		$order = '';
		if(!empty($orderclause)) {
			$order = self::orderClause($orderclause, '');
		}

		$limit = '';
		if(!empty($limitclause)) {
			$limit = self::limitClause($limitclause);
		}

		$sql = 'SELECT * FROM '.$sys_conf['db']['table_prefix'].$tablename.$where.$order.$limit;
		$req = $db->sql_query($sql);
		$error = $db->sql_error();
		if(intval($error['code']) === 0) {	// Aucune Erreur dtecte
			if(($line = $db->sql_numrows($req)) === 1) {
				$data = $db->sql_fetchrow($req);
				foreach($data as $key => $val) {
					$this->$key = $val;
				}
				return $line;
			} else {			// On a trop de ligne
				// On ne fait rien...
				return $line;
			}
		} else {				// Une erreur dans la query s'est droule
			throw new Exception('<b>SQL Error :</b> '. $error['message'] . '<br />'.$sql);
			return false;
		}
	}

	/**
	 * Utilise un where clause en fonction des paramtres obtenus
	 *
	 * @param string $table
	 * @param mixed $keyword
	 * @return string
	 */
	public static function whereClause($table, $keyword) {
		if(is_array($table)) {
			return ' WHERE '.self::_whereClause($table, $keyword);
		} else {
			return ' WHERE '.$table;
		}
	}

	/**
	 * Transforme la table en string ncessaire pour une clause where.
	 * Le paramtre $keyword permet de passer le AND ou OR ncessaire.
	 * Cette fonction est rcursive.
	 * Voir fill pour plus d'explications.
	 *
	 * @param mixed[] $table
	 * @param string $keyword
	 * @return string
	 */
	private static function _whereClause($table, $keyword) {
		$keywords = array('AND', 'OR');

		$ret = '(';
		foreach ($table as $key => $val) {
			if($ret !== '('){
				$ret .= ' ' . $keyword . ' ';
			}

			if(in_array(strtoupper($key), $keywords)) {
				$ret .= self::_whereClause($val, $key);
			} else {
				if(substr($key, 0, 1) === '!') {
					$ret .= '`' . substr($key, 1) . '`!=\'' . addslashes($val) . '\' ';
				} else {
					$ret .= '`' . $key . '`=\'' . addslashes($val) . '\' ';
				}
			}
		}
		$ret = trim($ret);
		$ret .= ')';
		return $ret;
	}

	/**
	 * Sort correctement la clause ORDER BY.
	 *
	 * @param mixed $table
	 * @return string
	 */
	public static function orderClause($table) {
		if(is_array($table)) {
			$display = ' ORDER BY';
			foreach($table as $key => $val) {
				$display .= ' '.$key.' '.$val.',';
			}
			$display = substr($display, 0, -1);
			return $display;
		} else {
			return ' ORDER BY '.$table;
		}
	}

	/**
	 * Sort correctement la clause LIMIT
	 *
	 * @param mixed $table
	 * @return string
	 */
	public static function limitClause($table) {
		if(is_array($table)) {
			$display = ' LIMIT ';
			if(count($table) === 2) {
				// Le premier est le nombre, le deuxime est l'index
				$display .= $table[1] . ',' . $table[0];
			} elseif (count($table) === 1) {
				$display .= $table[0];
			} else {
				$display = '';
			}
			return $display;
		} else {
			return ' LIMIT '.$table;
		}
	}

	/**
	 * Create primary array for filling.
	 *
	 * @param mixed $primaryclause
	 * @param array $data
	 */
	public static function primaryClauseWhere($primaryclause, $data) {
		if(!is_array($primaryclause))
			$primaryclause = array($primaryclause);

		$array['AND'] = array();
		$c = count($primaryclause);
		for($i = 0; $i < $c; $i++) {
			$array['AND'][$primaryclause[$i]] = $data[$primaryclause[$i]];
		}
		return $array;
	}

	/**
	 * Rcupre le nombre de ligne total dans une table suivant un where
	 *
	 * @param string $tablename
	 * @param mixed $whereclause
	 * @return int
	 */
	public static function getCount($tablename, $whereclause = '') {
		global $db, $sys_conf;

		$where = '';
		if(!empty($whereclause)) {
			$where = self::whereClause($whereclause, '');
		}

		$sql = 'SELECT COUNT(*) AS c FROM '.$sys_conf['db']['table_prefix'].$tablename.$where;
		$req = $db->sql_query($sql);
		$error = $db->sql_error();
		if(intval($error['code']) === 0) {	// Aucune Erreur dtecte
			$data = $db->sql_fetchrow();
			return $data['c'];
		} else {				// Une erreur dans la query s'est droule
			throw new Exception('<b>SQL Error :</b> '. $error['message'] . '<br />'.$sql);
			return false;
		}
	}

	/**
	 * Rcupre la limite  afficher en fonction de $_GET[$index] et de $max
	 *
	 * @param int $max
	 * @param string $index
	 * @return int
	 */
	public static function getLimit($max, $index = 'index') {
		if(isset($_GET[$index]) && intval($_GET[$index]) > 0) {
			return array($max, intval($_GET[$index]));
		} else {
			return array($max);
		}
	}

	/**
	 * Vrifie si la valeur $value est null. Si tel est le cas, on retourne simplement
	 * NULL, sinon on retourne la valeur entoure d'apostrophe
	 *
	 * @param mixed $value
	 * @return string
	 */
	public static function checkNull($value) {
		if($value === NULL || empty($value)) {
			return 'NULL';
		} else {
			return '\'' . addslashes($value) . '\'';
		}
	}

	/**
	 * Returns the current time, format depending on $nDate
	 *
	 * @param int $nDate
	 */
	public static function GetNow($nDate) {
		switch($nDate) {
			case self::DB_TIME_DATE:
				return date('Y-m-d');
			break;
			case self::DB_TIME_TIMESTAMP:
				return date('Y-m-d H:i:s');
			break;
			case self::DB_TIME_UNIX:
				return date('U');
			break;
		}
		return '';
	}
};
?>